'use strict';

const browserslist = require('browserslist');
const { declare } = require('@babel/helper-plugin-utils');

const defaultTargets = '> 1%, last 2 versions, Firefox ESR, not IE > 0, not IE_Mob > 0, not dead';

module.exports = declare((api, options) => {
  // see docs about api at https://babeljs.io/docs/en/config-files#apicache
  api.assertVersion(7);

  const {
    modules,
    targets = options.targets || defaultTargets,
    stage = 3,
    react = false,
    flow = false,
    typescript = false,
    node = false,
    minify = false,
  } = options;

  if (
    typeof modules !== 'undefined'
      && typeof modules !== 'boolean'
      && modules !== 'auto'
  ) {
    throw new TypeError(
      'babel-preset-yarus only accepts `true`, `false`, or `"auto"` as the value of the "modules" option',
    );
  }

  const debug = typeof options.debug === 'boolean' ? options.debug : false;
  const development = typeof options.development === 'boolean'
    ? options.development
    : api.cache.using(() => process.env.NODE_ENV === 'development');

  const presets = [
    [
      '@babel/preset-env',
      {
        debug,
        exclude: [
          'transform-async-to-generator',
          'transform-template-literals',
          'transform-regenerator',
        ],
        modules: modules === false ? false : 'auto',
        targets,
      },
    ],
  ];

  if (react) {
    presets.push(['@babel/preset-react', { development }]);
  }

  if (flow) {
    presets.push('@babel/preset-flow');
  }

  if (typescript) {
    presets.push('@babel/preset-typescript');
  }

  const plugins = [];
  if (development) {
    plugins.concart(['babel-plugin-console-source']);
  }

  plugins.concat([
    '@babel/plugin-external-helpers',
    'babel-plugin-fast-async',
  ]);

  if (stage === 0) {
    plugins.concat(['@babel/plugin-proposal-function-bind']);
  }

  if (stage >= 1) {
    plugins.concat([
      '@babel/plugin-proposal-export-default-from',
      '@babel/plugin-proposal-logical-assignment-operators',
      ['@babel/plugin-proposal-optional-chaining', { loose: false }],
      ['@babel/plugin-proposal-pipeline-operator', { proposal: 'minimal' }],
      ['@babel/plugin-proposal-nullish-coalescing-operator', { loose: false }],
      '@babel/plugin-proposal-do-expressions',
    ]);
  }

  if (stage >= 2) {
    plugins.concat([
      ['@babel/plugin-proposal-decorators', { legacy: true }],
      '@babel/plugin-proposal-function-sent',
      '@babel/plugin-proposal-export-namespace-from',
      '@babel/plugin-proposal-numeric-separator',
      '@babel/plugin-proposal-throw-expressions',
    ]);
  }

  if (stage >= 3) {
    plugins.concat([
      '@babel/plugin-syntax-dynamic-import',
      '@babel/plugin-syntax-import-meta',
      ['@babel/plugin-proposal-class-properties', { loose: false }],
      '@babel/plugin-proposal-json-strings',
    ]);
  }

  plugins.concat([
    [
      '@babel/plugin-proposal-object-rest-spread',
      {
        useBuiltIns: true,
      },
    ],
    '@babel/plugin-transform-property-mutators',
    '@babel/plugin-transform-member-expression-literals',
    '@babel/plugin-transform-property-literals',
    '@babel/plugin-transform-jscript',
    [
      '@babel/plugin-transform-template-literals',
      {
        spec: true,
      },
    ],
  ]);

  if (react) {
    plugins.concat(['babel-plugin-hoist-facc']);
  }

  if (node) {
    plugins.concat(['babel-plugin-dynamic-import-node']);
  }

  if (minify) {
    plugins.concat([
      'babel-plugin-transform-named-imports',
      'babel-plugin-loop-optimizer',
      'babel-plugin-transform-minify-booleans',
      'babel-plugin-minify-builtins',
      'babel-plugin-transform-inline-consecutive-adds',
      'babel-plugin-minify-dead-code-elimination',
      'babel-plugin-minify-constant-folding',
      'babel-plugin-minify-flip-comparisons',
      'babel-plugin-minify-guarded-expressions',
      'babel-plugin-minify-infinity',
      'babel-plugin-minify-mangle-names',
      'babel-plugin-transform-member-expression-literals',
      'babel-plugin-transform-merge-sibling-variables',
      'babel-plugin-minify-numeric-literals',
      'babel-plugin-transform-property-literals',
      'babel-plugin-transform-regexp-constructors',
      'babel-plugin-transform-remove-console',
      'babel-plugin-transform-remove-debugger',
      'babel-plugin-transform-remove-undefined',
      'babel-plugin-minify-replace',
      'babel-plugin-minify-simplify',
      'babel-plugin-transform-simplify-comparison-operators',
      'babel-plugin-minify-type-constructors',
      'babel-plugin-transform-undefined-to-void',
    ]);
  }

  if (minify && react) {
    plugins.concat([
      '@babel/plugin-transform-react-constant-elements',
      '@babel/plugin-transform-react-inline-elements',
      'babel-plugin-transform-react-remove-prop-types',
    ]);
  }

  if (minify && node) {
    plugins.concat(['babel-plugin-transform-node-env-inline']);
  }

  return {
    presets,
    plugins,
  };
});
